@@ -1,4 +1,4 @@
-__version__ = '0.1.4'
+__version__ = '0.1.5'
 
 ## Import packages
 import numpy as np
@@ -70,14 +70,19 @@ def find_vm_sd(channel_responses,plotting=False):
         plt.plot(make_gaussian_iter(180,np.arange(10,180)[np.argmin(losses)]))
         
     return np.arange(10,180)[np.argmin(losses)]
-def rotate_recon(features,recons):
-    aligned_crs = np.array([np.roll(recons[t,:],-features[t]) for t in range(len(features))])
-    return aligned_crs
+def rotate(l,dist):
+    # this is identical but faster than np.roll() for larger matrices 
+    l=l.tolist()
+    for i,d in enumerate(dist):
+        ll = l[i]
+        for _ in range(d):
+            ll.append(ll.pop(0))
+    return np.array(l)
 
 ## Main functions
 def IEM(trialbyvoxel,features,stim_max=180,is_circular=True,nfolds=10,
                       nchannels=9,channel_sd=None,channel_mus=None,
-                      plot_basis_set=False):
+                      plot_basis_set=False,calc_confidence=True):
     """
     Input trial by voxel matrix (or trial by electrode/component/etc.) and the array of presented stimuli and returns trial-by-trial predicted stimuli, confidence values (goodness of fits), and (non-aligned) reconstructions.
 
@@ -96,11 +101,13 @@ def IEM(trialbyvoxel,features,stim_max=180,is_circular=True,nfolds=10,
     nchannels : int, optional
         Number of basis channels in basis set. Note that iterative shifting will be applied such that reconstructions will always be of stim_max length, regardless of number of channels specified. Defaults to 9.
     channel_sd : float, optional
-        The standard deviation of each channel of the basis set. Defaults to standard deviation of a cosine raised to nchannels-1.
+        The standard deviation of each channel of the basis set. Defaults to standard deviation of a cosine raised to nchannels-1. Note that IEM() runs noticeably quicker if channel_sd is specified!
     channel_mus : array_like, optional
         The centers of each channel. Defaults to equally spaced channels. Due to iterative shifting, the centers of each channel are inconsequential if assuming equal spacing. However, if you do not want equally spaced channels (e.g., one channel is always farther apart from the other channels), then provide an array of the centers of each channel (in stimulus space). Note that iterative shifting is applied in all cases.
     plot_basis_set: bool, optional
         Optionally plot the basis set by setting to True. Only plots the first iteration of iterative shifting procedure. Defaults to False.
+    calc_confidence: bool, optional
+        Whether to return goodness of fits or not. If True, IEM() returns (predictions, confidences, recons, aligned_recons). If False, returns (predictions, recons, aligned_recons). Defaults to True.
 
     Returns
     -------
@@ -145,7 +152,7 @@ def IEM(trialbyvoxel,features,stim_max=180,is_circular=True,nfolds=10,
         sd = sd * stim_max/360
     else:
         sd = channel_sd
-    if channel_mus==None:
+    if np.all(channel_mus==None):
         est_mus = np.linspace(0,(stim_max-(stim_max/nchannels)),nchannels).astype(int)
     else:
         est_mus = channel_mus
@@ -165,7 +172,8 @@ def IEM(trialbyvoxel,features,stim_max=180,is_circular=True,nfolds=10,
 
     # define empty variables
     predictions = np.full(ntrials,np.nan)
-    confidences = np.full(ntrials,np.nan)
+    if calc_confidence:
+        confidences = np.full(ntrials,np.nan)
     recons = np.full((ntrials,stim_max),np.nan)
 
     kf = KFold(n_splits=nfolds)
@@ -188,23 +196,28 @@ def IEM(trialbyvoxel,features,stim_max=180,is_circular=True,nfolds=10,
             est_colors = np.argmax(generate_correlation_map(cr_tsts,
                                         make_gaussian_iter(np.arange(stim_max),
                                                    sd*np.ones(stim_max),stim_max)),axis=1)
-            precisions = np.array([sp.stats.linregress(make_gaussian_iter([est_colors[t]],
+            if calc_confidence:
+                precisions = np.array([sp.stats.linregress(make_gaussian_iter([est_colors[t]],
                                                    sd*np.ones(stim_max),stim_max).flatten(),
                                                    cr_tsts[t,:]).rvalue for t in range(len(tstf))])
         else:
             est_colors = np.argmax(generate_correlation_map(cr_tsts,
                                         make_noncirc_gaussian_iter(np.arange(stim_max),
                                                    sd*np.ones(stim_max),stim_max)),axis=1)
-            precisions = np.array([sp.stats.linregress(make_noncirc_gaussian_iter([est_colors[t]],
+            if calc_confidence:
+                precisions = np.array([sp.stats.linregress(make_noncirc_gaussian_iter([est_colors[t]],
                                                    sd*np.ones(stim_max),stim_max).flatten(),
                                                    cr_tsts[t,:]).rvalue for t in range(len(tstf))])
         predictions[test_index] = est_colors
-        confidences[test_index] = precisions
+        if calc_confidence:
+            confidences[test_index] = precisions
         recons[test_index,:] = cr_tsts
     # align the reconstructions at zero, such that ideally reconstruction is centered at 0 when plotted
-    aligned_recons = rotate_recon(features,recons)
-
-    return predictions, confidences, recons, aligned_recons
+    aligned_recons = rotate(recons,features)
+    if calc_confidence:
+        return predictions, confidences, recons, aligned_recons
+    else:
+        return predictions, recons, aligned_recons
 
 def permutation(features,stim_max=180,num_perm=5000,is_circular=True):
     """
